; Actual convolution function
function myconvolve_double, map, width, angle, aspect=aspect, $
	onephase=onephase, init=init

if n_elements(aspect) ne 1 then aspect=sqrt(2.0)
if keyword_set(onephase) then begin
  if strlowcase(strmid(onephase,0,1)) eq 'c' then phase=1 $
   else if strlowcase(strmid(onephase,0,1)) eq 's' then phase=-1 $
   else phase=0
endif else phase=0

common ftfields, xfeld, yfeld, radiusfeld, fourierfeld, fnx, fnxh, fny, fnyh

; FT map only if not done before
if keyword_set(init) then begin
  fwdfourier, map, fourierfeld
  ; field size
  ah=size(map)
  nx=ah[1]
  ny=ah[2]
  fnx=float(nx)
  fnxh=nx/2.0
  fny=float(ny)
  fnyh=ny/2.0
  yratio=fnxh/fnyh
  xfeld=fltarr(nx,ny)
  for i=0,nx-1 do xfeld[i,*]=replicate(i-fnxh,ny)
  yfeld=fltarr(nx,ny)
  for j=0,ny-1 do yfeld[*,j]=replicate(j-fnyh,nx)*yratio
  radiusfeld=sqrt(xfeld^2+yfeld^2)
endif
nmap=fnx*fny

; Now angle dependent part
pih=!pi/2.0
rotfeld=xfeld*cos(angle+pih)+yfeld*sin(angle+pih)

; define filter
a=width/(2.*!pi)
b=width*aspect
bp=b*2.0*!pi/fnx
ap=a*2.0*!pi/fnx
norm=exp(-b^2/(4.*a^2))
if phase ge 0 then begin
  realfeld=exp(-(radiusfeld*bp)^2/4.0) $
    *(cosh(rotfeld*bp^2/(2.0*ap))*exp(-bp^2/(4.*ap^2))-norm)*!pi*b^2
  tfeld=realfeld
endif 
if phase le 0 then begin
  imagfeld=exp(-(radiusfeld*bp)^2/4.0)*sinh(rotfeld*bp^2/(2.0*ap)) $
    *!pi*b^2*exp(-bp^2/(4.*ap^2))
  if phase eq 0 then tfeld=tfeld+imagfeld else tfeld=imagfeld
endif
; Avoid NaNs from underruns
invalid=where(~ finite(tfeld),count)
if (count gt 0) and (count lt nmap) then tfeld[invalid]=0.0

; Convert back
newfeld=fourierfeld*tfeld
bckfourier, newfeld, output1

; output - should be correct in absolute scales
return, (abs(output1)^2)*nmap^2/b^3*aspect

end

function scaleconvolve, map, b, aspect=aspect, onephase=onephase

; angular grid
nang=180
angarr=findgen(nang)/nang*!pi

; First call initilizing FT of main field
miarr=myconvolve_double(map,b,0.0,aspect=aspect,onephase=onephase,/init) &$
marealarr=miarr
maimagarr=miarr*0.0

; all other angles
for i=1,nang-1 do begin &$
  rpart=cos(2*angarr[i]) &$
  ipart=sin(2*angarr[i]) &$
  output=myconvolve_double(map,b,angarr[i],aspect=aspect,onephase=onephase) &$
  miarr=miarr+output &$
  marealarr=marealarr+output*rpart &$
  maimagarr=maimagarr+output*ipart &$
endfor
miarr=miarr/nang
marealarr=marealarr/nang
maimagarr=maimagarr/nang
phiarr=atan(maimagarr,marealarr)/2.0

return, [[[miarr]],[[sqrt(marealarr^2+maimagarr^2)]],[[phiarr]]]
end

